import sys, time
import pygame

import player, level, camera, draw
from util import SW, SH, init_objects, load_image, load_sounds

# based on http://stackoverflow.com/questions/14700889/pygame-level-menu-states
class Scene(object):
    def __init__(self):
        self.drawer = draw.Draw(SW, 14)
        self.text_to_render = ''

    def render(self, screen):
        raise NotImplementedError

    def update(self):
        raise NotImplementedError

    def handle_events(self, events):
        raise NotImplementedError

class SceneMananger(object):

    def __init__(self, screen, level):
        self.c_level = level
        self.screen = screen
        self.ticks = 0
        self.go_to(IntroScene())

    def go_to(self, scene, lp=False):
        self.screen.fill((0,0,0))
        pygame.display.update()
        if lp: 
            time.sleep(0.3)      
        self.scene = scene
        self.scene.manager = self


class IntroScene(Scene):
    def __init__(self):
        self.slides = []
        for i in range(0,6):
            self.slides.append(load_image('slide'+str(i)+'.png'))
        self.inx = 0
        self.ticks = 0

    def update(self, keys):
        pass

    def handle_events(self, keys):
        if keys[pygame.K_SPACE]:
            self.manager.go_to(TitleScene())

    def render(self, screen):
        screen.blit(self.slides[self.inx], (0,0))
        self.ticks += 1
        if self.ticks > 360:
            self.ticks = 0
            self.inx += 1

        if self.inx == 6:
            self.manager.go_to(TitleScene())

class TitleScene(Scene):

    def __init__(self):
        super(TitleScene, self).__init__()
        img = load_image('title.png')
        self.title = pygame.sprite.Sprite()
        self.title.image = img 
        self.title.rect = self.title.image.get_rect()

        self.menu = ('Press Enter to start', 'Arrows to move, Space to jump, S to fire,', 'Esc to Quit')

        self.line_height = 20
        self.start_y = SW - 200 

    def update(self, keys):
        pass

    def render(self, screen):
        screen.blit(self.title.image, self.title.rect)
        self.drawer.draw_text(screen, self.menu, self.start_y)

    def handle_events(self, keys):
        if keys[pygame.K_ESCAPE]:
            sys.exit()

        if keys[pygame.K_RETURN]:
            self.manager.go_to(GameScene(self.manager.c_level))




class GameScene(Scene):

    def __init__(self, c_level, player=False):
        super(GameScene, self).__init__()
        self.c_level = c_level
        self.level = level.Level(c_level)        
        self.entities, self.player, self.o_b = init_objects(self.level.objects, self)
        if player:
            self.player.lifes = player.lifes
            self.player.hp = player.hp

        self.camera = camera.Camera(self.player.rect, SW, SH)
        self.fx_channel = pygame.mixer.Channel(1)
        self.sounds = ['pickup', 'explotion5']
        self.sounds = load_sounds(self.sounds)

    def update_entities(self, entities, keys):
        for e in entities:
            if isinstance(e, player.Player):
                self.player.update(keys, self.level, self.o_b)
                self.player.manager = self.manager
            else:
                e.update(self.player)

    def handle_events(self, keys):
        if keys[pygame.K_ESCAPE]:
            self.manager.go_to(TitleScene())

    def update(self, keys):
        self.text_to_render = ''
        self.update_entities(self.entities, keys)
        self.camera.update(self.player.rect, self.level)

    def render(self, screen):
        screen.fill((90,120,120))
        self.drawer.draw_level(screen, self.level, self.camera)
        self.drawer.draw_entities(screen, self.entities, self.camera)
        self.drawer.draw_info(screen, screen.get_rect(), self.player)
        if self.text_to_render:
            self.drawer.draw_text(screen, self.text_to_render, 200)
        pygame.display.update(self.camera)

class LevelComplete(Scene):
    def __init__(self, next_level, player):
        super(LevelComplete, self).__init__()
        self.player = player
        self.level = next_level
        self.text = ['Level complete','Press enter to continuie']


    def handle_events(self, keys):
        if keys[pygame.K_RETURN]:
            self.manager.go_to(GameScene(self.level, self.player))

    def update(self, keys):
        pass

    def render(self, screen):
        screen.fill((0,0,0))
        self.drawer.draw_info(screen, screen.get_rect(), self.player)
        self.drawer.draw_text(screen, self.text, SW - 200 )

class LostLife(Scene):
    def __init__(self, level, player):
        super(LostLife, self).__init__()
        self.player = player
        self.player.hp = 100
        self.level = level
        self.text = ['Keep Fighting','Lifes left {}'.format(player.lifes), 'Press enter to continuie or Esc to Quit']


    def handle_events(self, keys):
        if keys[pygame.K_RETURN]:
            self.manager.go_to(GameScene(self.level, self.player))

    def update(self, keys):
        pass

    def render(self, screen):
        screen.fill((0,0,0))
        self.drawer.draw_text(screen, self.text, SW/2 - 100 )


class GameOver(Scene):
    def __init__(self, level, player):
        super(GameOver, self).__init__()
        self.player = player
        self.player.hp = 100
        self.level = level
        self.text = ['Game over', 'Thanks to play','Press Space to continuie or Esc to Quit']


    def handle_events(self, keys):
        if keys[pygame.K_SPACE]:
            self.manager.level = 'data/maps/jungle.tmx'
            self.manager.go_to(TitleScene())

        if keys[pygame.K_ESCAPE]:
            sys.exit()

    def update(self, keys):
        pass

    def render(self, screen):
        screen.fill((0,0,0))
        self.drawer.draw_text(screen, self.text, SW/2 - 100 )


class WonScene(Scene):
    def __init__(self, level, player):
        super(WonScene, self).__init__()
        self.player = player
        self.player.hp = 100
        self.level = level
        self.text = ['congratulations you won', 'Thanks to play','Press Esc to Quit', '', 'entry in PyWeek #18 - ANDarms']


    def handle_events(self, keys):
        if keys[pygame.K_SPACE]:
            self.manager.level = 'data/maps/jungle.tmx'
            self.manager.go_to(TitleScene())

        if keys[pygame.K_ESCAPE]:
            sys.exit()

    def update(self, keys):
        pass

    def render(self, screen):
        screen.fill((0,0,0))
        self.drawer.draw_text(screen, self.text, SW/2 - 200 )